package cc.fascinated.bat.features.moderation.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.moderation.punish.Punishment;
import cc.fascinated.bat.features.moderation.punish.PunishmentProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.UserService;
import com.github.ygimenez.method.Pages;
import com.github.ygimenez.model.InteractPage;
import com.github.ygimenez.model.Page;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(
        name = "punish-history",
        description = "View the punishment history of a member",
        requiredPermissions = Permission.MANAGE_SERVER
)
public class PunishHistoryCommand extends BatCommand {
    private static final int PUNISHMENTS_PER_PAGE = 10;
    private final UserService userService;

    @Autowired
    public PunishHistoryCommand(@NonNull UserService userService) {
        this.userService = userService;
        super.addOptions(
                new OptionData(OptionType.USER, "member", "The member you want to view the punishment history of", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping memberOption = event.getOption("member");
        assert memberOption != null;
        BatUser targetUser = userService.getUser(memberOption.getAsUser().getId());
        if (targetUser == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("User not found")
                    .build()
            ).queue();
            return;
        }

        PunishmentProfile profile = guild.getPunishmentProfile();
        List<Punishment> punishments = profile.getPunishments(targetUser);

        List<Page> pages = new ArrayList<>();
        int totalPages = (int) Math.ceil((double) punishments.size() / PUNISHMENTS_PER_PAGE);

        if (totalPages >= 1) {
            for (int i = 0; i < totalPages; i++) {
                int fromIndex = i * PUNISHMENTS_PER_PAGE;
                int toIndex = Math.min(fromIndex + PUNISHMENTS_PER_PAGE, punishments.size());
                List<Punishment> subList = punishments.subList(fromIndex, toIndex);
                DescriptionBuilder history = new DescriptionBuilder("Punishment history for %s".formatted(targetUser.getDiscordUser().getAsMention()));

                for (Punishment punishment : subList) {
                    BatUser issuer = userService.getUser(punishment.getIssuer());
                    history.appendLine("> **[%s%s]** <t:%s:R> Reason: `%s` | Issuer: %s%s".formatted(
                            punishment.getEndDate() != null ? "TEMP-" : "",
                            punishment.getType().name(),
                            punishment.getIssuedDate().toInstant().getEpochSecond(),
                            punishment.getReason() == null ? "N/A" : punishment.getReason(),
                            issuer.getDiscordUser().getAsMention(),
                            punishment.isExpired() ? " **(Expired)**" : ""
                    ), false);
                }

                pages.add(InteractPage.of(EmbedUtils.successEmbed()
                        .setDescription(history.build())
                        .setFooter("Page %d/%d".formatted(i + 1, totalPages))
                        .build()));
            }
        } else {
            pages.add(InteractPage.of(EmbedUtils.successEmbed()
                    .setDescription("No punishments found for %s".formatted(targetUser.getDiscordUser().getAsMention()))
                    .build()));
        }

        event.replyEmbeds((MessageEmbed) pages.get(0).getContent()).queue(success -> {
            if (pages.size() < 2) { // If there is only one page, we don't need to paginate
                return;
            }
            success.retrieveOriginal().queue(message -> {
                Pages.paginate(message, pages, true, (interactedUser) -> interactedUser.getIdLong() == user.getDiscordUser().getIdLong());
            });
        });
    }
}
